#!/usr/bin/env python

__version__ = '$Revision: 7470 $'.split()[1]
__date__ = '$Date: 2007-11-06 10:31:44 -0500 (Tue, 06 Nov 2007) $'.split()[1]
__author__ = 'Kurt Schwehr'

__doc__='''

Simple serial port logger.  Log times added to the stream are in UTC
seconds since the Epoch (UTC).

@todo: Optional WWW and UDP transmitting
@todo: Clean way to shut down

@requires: U{epydoc<http://epydoc.sourceforge.net/>} > 3.0alpha3
@requires: U{pyserial<http://pyserial.sourceforge.net/>}
@author: '''+__author__+'''
@version: ''' + __version__ +'''
@var __date__: Date of last svn commit
@undocumented: __version__ __author__ __doc__ parser defaultPort defaultPorts speeds
@status: under development
@license: GPL v2
@since: 2007-Feb-17
@todo: Add a line for STOP LOGGING when a log file is closed
@todo: Add lines at start of logging for computer name, user name, and ???
'''

import sys
import os
import time
import serial

def createDaemon():
   """nohup like function to detach from the terminal"""

   try:
      pid = os.fork()
   except OSError, e:
      raise Exception, "%s [%d]" % (e.strerror, e.errno)

   if (pid == 0):
      # The first child.
      os.setsid()

      try:
         pid = os.fork()	# Fork a second child.
      except OSError, e:
         raise Exception, "%s [%d]" % (e.strerror, e.errno)

      if (pid != 0):
         os._exit(0)	# Exit parent (the first child) of the second child.

   else:
      os._exit(0)	# Exit parent of the first child.

   import resource		# Resource usage information.
   maxfd = resource.getrlimit(resource.RLIMIT_NOFILE)[1]
   if (maxfd == resource.RLIM_INFINITY):
       maxfd = 1024
  
   # Iterate through and close all file descriptors.
   if True:
       for fd in range(0, maxfd):
           try:
               os.close(fd)
           except OSError:	# ERROR, fd wasn't open to begin with (ignored)
               pass

   # Send all output to /dev/null - FIX: send it to a log file
   os.open('/dev/null', os.O_RDWR)
   os.dup2(0, 1)
   os.dup2(0, 2)

   return(0)


def dateStr():
    '''
    String representing the day so that it sorts correctly
    @return: yyyy-mm-dd
    @rtype: str
    '''
    t=time.gmtime()
    d='%04d-%02d-%02d' % t[:3]
    return d


def start(options):
    '''
    Do the logging
    '''
    verbose = options.verbose

    ser = serial.Serial(options.port,options.baud,timeout=options.timeout)
    
    logfile=options.log_prefix+dateStr()
    currentDay = time.gmtime()[2]
    if not options.daemonMode:
       print 'opening logfile:',logfile
    log=open(logfile,'a')
    log.write('# START LOGGING UTC seconds since the epoch: '+str(time.time())+'\n')
    log.write('# SPEED:       '+str(options.baud)+'\n')
    log.write('# PORT:        '+str(options.port)+'\n')
    log.write('# TIMEOUT:     '+str(options.timeout)+'\n')
    log.write('# STATIONID:   '+str(options.stationId)+'\n')
    log.write('# DAEMON MODE: '+str(options.daemonMode)+'\n')
    log.flush()

    while True:
        # Handle date rollover
        if currentDay!=time.gmtime()[2]:
            currentDay=time.gmtime()[2]
            log.close()
            logfile=options.log_prefix+dateStr()
            log=open(logfile,'a')
            log.write('# START LOGGING UTC seconds since the epoch: '+str(time.time())+'\n')
            log.write('# SPEED: '+str(options.baud)+'\n')
            log.write('# PORT:  '+str(options.port)+'\n')

        line = ser.readline().strip()
        timestamp = time.time()
	if len(line)==0:
            if verbose: print '# --- No data ---'
            if options.mark:
                log.write('# MARK: '+str(timestamp)+'\n')
                if options.flush: log.flush()
	    continue
	elif verbose: 
            if options.uscgFormat:
                if options.stationId != None:
                    print line+','+options.stationId+','+str(timestamp)
                else:
                    print line+','+str(timestamp)
            else:
                print line

	if len(line)>0:
            if options.uscgFormat:
                log.write(line)
                if options.stationId != None: log.write(',r'+options.stationId)
                log.write(','+str(timestamp)+'\n')
            else:
                log.write('# ' +str(timestamp)+'\n')
                log.flush() # FIX: better not to flush so writing to disk less often
                log.write(line+'\n')

        elif options.uscgFormat:
            log.write('# ' +str(timestamp)+'\n') # allow detection of the station being active

        if options.flush: log.flush()


######################################################################
       
if __name__=='__main__':
    from optparse import OptionParser

    parser = OptionParser(usage="%prog [options]",
                            version="%prog "+__version__)

    defaultPorts = {'Darwin':'/dev/tty.KeySerial1', 'Linux':'/dev/ttyS0'}
    defaultPort = '/dev/ttyS0'
    if os.uname()[0] in defaultPorts: defaultPort=defaultPorts[os.uname()[0]]

    parser.add_option('-d','--daemon',dest='daemonMode',default=False,action='store_true'
                      ,help='Detach from the terminal and run as a daemon service.'
                      +'  Returns the pid. [default: %default]')
    parser.add_option('--pid-file',dest='pidFile',default=None
                      ,help='Where to write the process id when in daemon mode')

    parser.add_option('-p','--port',dest='port',type='string', default=defaultPort,
                      help='What serial port device to option [default: %default]')

    #speeds=serial.baudEnumToInt.keys()
    #speeds.sort()
    speeds=[ #0, 50, 75, 110,
            #134, 150, 200, 
            300, 
            600, 
            1200, 
            1800, 
            2400, 
            4800, 
            9600,
            19200, 38400, 57600, 115200, 230400
            ]

    speeds=[str(s) for s in speeds]

    parser.add_option('-b','--baud',dest='baud',
                      choices=speeds, type='choice', default='38400',
                      help='Port speed [default: %default].  Choices: '+', '.join(speeds))

    parser.add_option('-F','--no-flush',dest='flush',default=True,action='store_false',
                      help='Do not flush after each write')

    parser.add_option('-l','--log-prefix',dest='log_prefix',type='string',default='log.',
                      help='prefix before date of the log file [default: %default]')

    parser.add_option('-m','--mark-timeouts',dest='mark',default=False,action='store_true',
                      help='Mark the timeouts in the log file')

    parser.add_option('-t','--timeout',dest='timeout',type='float', default='300',
                      help='Number of seconds to timeout after if no data [default: %default]')

    parser.add_option('-v','--verbose',dest='verbose',default=False,action='store_true',
                      help='Make the test output verbose')

    parser.add_option('-u','--uscg-format',dest='uscgFormat',default=False,action='store_true',
                      help='Switch to the USCG N-AIS format with ",station,UTC sec" at the end of each line')

    parser.add_option('-s','--station-id',dest='stationId',type='string',default=None,
                      help='If uscg format is selected, you can specify a station id to'
                      +' put as ",r" before the timestamp  [default: %default]')

    (options,args) = parser.parse_args()

    options.baud = int(options.baud)

    if options.daemonMode:
        createDaemon()
        if options.pidFile!=None:
            open(options.pidFile,'w').write(str(os.getpid())+'\n')

    start(options)

